#include "Environment.h"
#include "GvsPreprocessor.h"
#include "GlRenderer.h"
#include "VssRay.h"
#include "ViewCellsManager.h"



namespace GtpVisibilityPreprocessor
{
  

GvsPreprocessor::GvsPreprocessor(): Preprocessor(), mSamplingType(0)
{
	Environment::GetSingleton()->GetIntValue("RenderSampler.samples", mSamples);
	//Environment::GetSingleton()->GetIntValue("Render.samples", mTotalSamples);

	cout << "number of render samples: " << mSamples << endl;
}


int GvsPreprocessor::HandleRay(VssRay &ray)
{
	int sampleContributions = 0;

	if (mViewCellsManager->ComputeSampleContribution(ray, true, false))
	{
		return AdaptiveBorderSampling(ray);
	}
	else
	{
		return ReverseSampling(ray);
	}
}


int GvsPreprocessor::AdaptiveBorderSampling(VssRay &ray)
{
	// TODO
	return 0;
}


int GvsPreprocessor::ReverseSampling(VssRay &ray)
{
	// TODO
	return 0;
}

int GvsPreprocessor::CastAvsSamples(const int samplesPerPass, 
									const int sampleType, 
									RayQueue &passSamples)
{
	SimpleRayContainer simpleRays;
	const long startTime = GetTime();

	GenerateRays(samplesPerPass, sampleType, simpleRays);
	Debug << "generated " <<  samplesPerPass << " samples in " << TimeDiff(startTime, GetTime()) * 1e-3 << " secs" << endl;

	VssRayContainer samples;

	// cast the rays and optain hitpoints with geometry
	CastRays(simpleRays, samples);
	Debug << "cast " <<  samplesPerPass << " samples in " << TimeDiff(startTime, GetTime()) * 1e-3 << " secs" << endl;

	// add to ray queue
	VssRayContainer::const_iterator vit, vit_end = samples.end();

	for (vit = samples.begin(); vit != vit_end; ++ vit)
	{
		mRayQueue.push(*vit);
	}

	return (int)mRayQueue.size();
}


int GvsPreprocessor::Pass(const int passSamples)
{
	int castSamples = 0;
	const int uniformSamples = 1000;

	while (castSamples < passSamples) 
	{
		// Ray queue empty => 
		// cast a number of uniform samples to fill ray Queue
		CastAvsSamples(uniformSamples, mSamplingType, mRayQueue);
		castSamples += Pass(passSamples);
	}

	return castSamples;
}


int GvsPreprocessor::RunSampling()
{
	int castSamples = 0;

	while (!mRayQueue.empty()) 
	{
		// handle next ray
		VssRay *ray = mRayQueue.top();
		mRayQueue.pop();
		
		castSamples += HandleRay(*ray);
	}

	return castSamples;
}


bool
GvsPreprocessor::ComputeVisibility()
{
	Randomize(0);
	const long startTime = GetTime();
	
	cout<<"Gvs Preprocessor started\n"<<flush;
	
	/// construct the view cells first
	ConstructViewCells();

	int samples = 0;

	while (samples < mTotalSamples) 
	{
		samples += Pass(mSamplesPerPass);
		
		cout<<"+";
		
		// mVssRays.PrintStatistics(mStats);
		//mStats << "#Time\n" << TimeDiff(startTime, GetTime())*1e-3 << endl
		//	   << "#TotalSamples\n" << samples << endl;

		mViewCellsManager->PrintPvsStatistics(mStats);
		// ComputeRenderError();
	}

	return true;
}

}
